<?php
/**
 * Groups API Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Api');

class GroupsController extends ApiController {
	var $name = 'Groups';
	var $components = array('RequestHandler');
	var $uses = array('Mygroup' , 'Group' , 'GroupSphost' , 'AddGroup' , 'SpHost', 'AutoGroup');

	function beforeRender(){
		Configure::write('debug',0);
		$this->RequestHandler->setContent('json');
		$this->RequestHandler->respondAs('application/json; charset=UTF-8');
		parent::beforeRender();
	}

	/**
	 * excute group API
	 * @param $find @me: 接続している全てのグループ
	 *              SPC ID|SPC entityID: 検索対象のSPコネクタIDまたはSPコネクタentityID
	 * 使用例)  https://xxxx/api/groups/@me
	 *          https://xxxx/api/groups/meatmail
	 *          https://xxxx/api/groups/https%3A%2F%2Fcg.gakunin.jp%2Fsp%2Fmeatmail
	 */
	function index($find = null) {
		$controller = $this->params['controller'];
		$lang = array_key_exists('lang', $this->params['url']) ? $this->params['url']['lang'] : 'en';
		$this->apiInfoLog('groups  find='.$find . ' url='.Router::url(null, true) . ' lang='.$lang);

		if ($find == null) {
			$this->apiErrorLog('not find parameter(find)');
			$this->cakeError('jsonBadRequest');
			return;
		}

		if ($find != "@me") {
			// groupKey を URL で分割する
			// %2f 文字を含む Parameter は CakePHP では取得できない為、URL から直接取り出す
			$groupKey = str_replace($controller, '', strstr(Router::url(null, true), $controller));
			$this->apiInfoLog('groupKey url is ' . $groupKey);
			if (($groupKey === '') || ($groupKey === '/')){
				$this->apiErrorLog('not find group id(key)');
				$this->cakeError('jsonNotFound');
				return;
			}

			$groupUrl = $groupKey;
			$groupKeyParts = explode('/', $groupKey);
			$find = array_pop($groupKeyParts);
		}

		// グループ一覧を作成
		$groupList = $this->getGroupList($find);

		$this->apiInfoLog('groupList for ' . $find . ' is follows.');
		foreach($groupList as $i)
			$this->apiInfoLog("  ".$i['Group']['id']." , ".$i['Group']['name']);

		// グループ内メンバ数の取得
		//$members = $this->getMembers($groupList);
		//$this->apiInfoLog('members count is ' . count($members));

		// 結果を返す
		$result['totalResults'] = count($groupList);
		$result['entry'] = array();
		foreach ($groupList as $group) {
			$entry = array('id'=> $this->getGroupId($group['Group']['group_key']));
			$entry_name = $this->getValue($group['Group'],'name',$lang);
			if (!empty($entry_name))
				$entry = array_merge($entry , array('title'=>$entry_name));
			$entry_desc = $this->getValue($group['Group'],'introduction',$lang);
			if (!empty($entry_desc))
				$entry = array_merge($entry , array('description'=>$entry_desc));
			$entry = array_merge($entry , array('map_totalMembers'=> count($this->getMembers($group))));
			array_push($result['entry'], $entry);
		}
		$this->set('result', $result);
	}


	private function getGroupId($groupKey){
		$url = Configure::read("entityID");
		$url = substr($url , 0 , strpos($url,'/idp'));
		return $url . "/gr/$groupKey";
		//return str_replace("/api","",Router::url("/gr/$groupKey",true));
	}

	/**
	 * 言語を指定して値を取得する　
	 */
	private function getValue($group , $key , $lang){
		$vals = array();
		if (!empty($group[$key . '_en']))
			$vals = array_merge($vals , array('en' => $group[$key . '_en']));
		if (!empty($group[$key]))
			$vals = array_merge($vals , array('ja' => $group[$key]));

		$val = (array_key_exists($lang,$vals) ? $vals[$lang] : array_shift($vals));
		return (!empty($val) ? $val : null);
	}

	/**
	 * Group 内メンバを取得する
	 */
	private function getMembers($group){

		$groupIdList = $this->getBottomGroups($group['Group']['id']);
		array_push($groupIdList,$group['Group']['id']);

		// Group 内メンバを取得する
		$members = $this->Mygroup->find('all', array(
			'fields' => array('DISTINCT Mygroup.account_id'),
			'conditions' => array(
				'groupid' => $groupIdList,
				'admin' => array(0,2))));
		return $members;
	}

	/**
	 * 下位グループを再帰的に取得する
	 */
	private function getBottomGroups($groupId){
		$bottomGroupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id'),
			'conditions' => array('top_id' => $groupId)));
		$groupIdList = $bottomGroupIdList;
		foreach($bottomGroupIdList as $i)
			$groupIdList = array_merge($groupIdList,$this->getBottomGroups($i));
		return $groupIdList;
	}

	/**
	 * SP からグループIDの一覧を取得する
	 */
	private function getGroupList($find){
		$groupIdList = array();

		// SP を取得
		$spHost = $this->getSpHost();

		// SP が利用している SP コネクタを取得
		// Group の一覧もここから取得する
		$spcIdList = $this->GroupSphost->find('list' , array(
			'fields' => array('group_id', 'group_id'),
			'conditions' => array('sp_id' => $spHost['SpHost']['id'])));

		if ($find != "@me") {
			$spcId = $this->Group->find('list', array(
				'fields' => array('group_key', 'id'),
				'conditions' => array('group_key' => $find)));
			if (empty($spcId)) {
				$this->apiErrorLog('not found SPC:' . $find);
				$this->cakeError('jsonNotFound');
				return;
			}

			if (array_key_exists($spcId[$find], $spcIdList)) {
				$spcIdList = $spcId;
			} else {
				$this->apiErrorLog('SP:' . $spHost['SpHost']['entityid'] . ' doesn\'t have joined SPC:' . $find);
				$this->cakeError('jsonCannotAccess');
				return;
			}
		}

		// SP の下位グループを取得(1階層のみ取得)
		$groupIdList = $this->AddGroup->find('list', array(
			'fields' => array('bottom_id'),
			'conditions' => array('top_id' => $spcIdList)));
		$groupIdList = array_merge($groupIdList , $spcIdList);

		// IdP機関グループを取得
		$idpGroupIdList = $this->AutoGroup->find('list', array(
			'fields' => array('group_id'),
			'conditions' => array('idp' => 'true')));

		// SP コネクタ、およびIdP機関グループ以外のグループを取得
		$groupIdList = $this->Group->find('all', array(
			'fields' => array('id','group_key','name','name_en','introduction','introduction_en','public'),
			'conditions' => array(
				'active' => 0 , 'sp' => 0,
				'id' => $groupIdList, 'NOT' => array('id' => $idpGroupIdList))));

		return $groupIdList;
	}
}
?>
